@@ -71,12 +71,14 @@ gdict_app_finalize (GObject *object)
   		   NULL);
   g_slist_free (app->windows);
 
+  g_list_foreach (app->words,
+		  (GFunc) g_free,
+		  NULL);
+  g_list_free (app->words);
+
   if (app->database)
     g_free (app->database);
 
-  if (app->word)
-    g_free (app->word);
-  
   if (app->source_name)
     g_free (app->source_name);
   
@@ -94,26 +96,8 @@ gdict_app_class_init (GdictAppClass *klass)
 static void
 gdict_app_init (GdictApp *app)
 {
-  gchar *data_dir;
-  
   app->windows = NULL;
   app->current_window = NULL;
-  
-  /* create the data directory inside $HOME, if it doesn't exist yet */
-  data_dir = g_build_filename (g_get_home_dir (),
-  			       ".gnome2",
-  			       "gnome-dictionary",
-  			       NULL);
-  
-  if (g_mkdir (data_dir, 0700) == -1)
-    {
-      if (errno != EEXIST)
-        g_warning ("Unable to create the data directory '%s': %s",
-		   data_dir,
-		   strerror (errno));
-    }
-  
-  g_free (data_dir);
 }
 
 static gboolean
@@ -175,30 +159,47 @@ gdict_window_created_cb (GdictWindow *parent,
 static void
 gdict_create_window (GdictApp *app)
 {
-  GtkWidget *window;
-  
-  window = gdict_window_new (singleton->loader,
-		  	     singleton->source_name,
-			     singleton->word);
-  
-  g_signal_connect (window, "created",
-  		    G_CALLBACK (gdict_window_created_cb), app);
-  g_signal_connect (window, "destroy",
-  		    G_CALLBACK (gdict_window_destroy_cb), app);
-  
-  app->windows = g_slist_prepend (app->windows, window);
-  app->current_window = GDICT_WINDOW (window);
-  
-  gtk_widget_show (window);
-}
+  GList *l;
 
-static void
-lookup_start_cb (GdictContext *context,
-		 gpointer      user_data)
-{
-  GdictApp *app = GDICT_APP (user_data);
+  if (!singleton->words)
+    {
+      GtkWidget *window;
 
-  g_print (_("Definitions for \"%s\":\n\n"), app->word);
+      window = gdict_window_new (singleton->loader,
+				 singleton->source_name,
+				 NULL);
+      g_signal_connect (window, "created",
+		        G_CALLBACK (gdict_window_created_cb), app);
+      g_signal_connect (window, "destroy",
+		        G_CALLBACK (gdict_window_destroy_cb), app);
+  
+     app->windows = g_slist_prepend (app->windows, window);
+     app->current_window = GDICT_WINDOW (window);
+  
+     gtk_widget_show (window);
+
+     return;
+   }
+      
+  for (l = singleton->words; l != NULL; l = l->next)
+    {
+      gchar *word = (gchar *) l->data;
+      GtkWidget *window;
+
+      window = gdict_window_new (singleton->loader,
+		      		 singleton->source_name,
+				 word);
+      
+      g_signal_connect (window, "created",
+		        G_CALLBACK (gdict_window_created_cb), app);
+      g_signal_connect (window, "destroy",
+		        G_CALLBACK (gdict_window_destroy_cb), app);
+  
+      app->windows = g_slist_prepend (app->windows, window);
+      app->current_window = GDICT_WINDOW (window);
+  
+      gtk_widget_show (window);
+    }
 }
 
 static void
@@ -206,8 +207,14 @@ definition_found_cb (GdictContext    *context,
 		     GdictDefinition *definition,
 		     gpointer         user_data)
 {
-  /* Translators: source name first, then definition */
-  g_print (_("From %s:\n%s\n"),
+  /* Translators: the first is the word found, the second is the
+   * database name and the last is the definition's text; please
+   * keep the new lines. */
+  g_print (_("Definition for '%s'\n"
+	     "  From '%s':\n"
+	     "\n"
+	     "%s\n"),
+           gdict_definition_get_word (definition),
 	   gdict_definition_get_database (definition),
 	   gdict_definition_get_text (definition));
 }
@@ -218,6 +225,20 @@ error_cb (GdictContext *context,
 	  gpointer      user_data)
 {
   g_print (_("Error: %s\n"), error->message);
+
+  gtk_main_quit ();
+}
+
+static void
+lookup_end_cb (GdictContext *context,
+	       gpointer      user_data)
+{
+  GdictApp *app = GDICT_APP (user_data);
+
+  app->remaining_words -= 1;
+
+  if (app->remaining_words == 0)
+    gtk_main_quit ();
 }
 
 static void
@@ -225,9 +246,9 @@ gdict_look_up_word_and_quit (GdictApp *app)
 {
   GdictSource *source;
   GdictContext *context;
-  GError *err;
+  GList *l;
   
-  if (!app->word)
+  if (!app->words)
     {
       g_print (_("See gnome-dictionary --help for usage\n"));
 
@@ -254,27 +275,34 @@ gdict_look_up_word_and_quit (GdictApp *app)
   context = gdict_source_peek_context (source);
   g_assert (GDICT_IS_CONTEXT (context));
 
-  g_signal_connect (context, "lookup-start",
-		    G_CALLBACK (lookup_start_cb), app);
   g_signal_connect (context, "definition-found",
 		    G_CALLBACK (definition_found_cb), app);
-  g_signal_connect (context, "lookup-end",
-		    G_CALLBACK (gtk_main_quit), NULL);
   g_signal_connect (context, "error",
 		    G_CALLBACK (error_cb), app);
+  g_signal_connect (context, "lookup-end",
+		    G_CALLBACK (lookup_end_cb), app);
 
-  err = NULL;
-  gdict_context_define_word (context,
-		  	     app->database,
-		  	     app->word,
-			     &err);
-  if (err)
+  app->remaining_words = 0;
+  for (l = app->words; l != NULL; l = l->next)
     {
-      g_warning (_("Error while looking up the definition of \"%s\":\n%s"),
-		 app->word,
-		 err->message);
+      gchar *word = (gchar *) l->data;
+      GError *err = NULL;
 
-      g_error_free (err);
+      app->remaining_words += 1;
+
+      gdict_context_define_word (context,
+		      		 app->database,
+				 word,
+				 &err);
+
+      if (err)
+	{
+          g_warning (_("Error while looking up the definition of \"%s\":\n%s"),
+		     word,
+		     err->message);
+
+	  g_error_free (err);
+	}
     }
 
   gtk_main ();
@@ -285,6 +313,101 @@ gdict_look_up_word_and_quit (GdictApp *app)
   exit (0);
 }
 
+/* create the data directory inside $HOME, if it doesn't exist yet */
+static gboolean
+gdict_app_create_data_dir (void)
+{
+  gchar *data_dir_name;
+  
+  data_dir_name = g_build_filename (g_get_home_dir (),
+  				    ".gnome2",
+  				    "gnome-dictionary",
+  				    NULL);
+  
+  if (g_mkdir (data_dir_name, 0700) == -1)
+    {
+      /* this is weird, but sometimes there's a "gnome-dictionary" file
+       * inside $HOME/.gnome2; see bug #329126.
+       */
+      if ((errno == EEXIST) &&
+          (g_file_test (data_dir_name, G_FILE_TEST_IS_REGULAR)))
+        {
+          gchar *backup = g_strdup_printf ("%s.pre-2-14", data_dir_name);
+	      
+	  if (g_rename (data_dir_name, backup) == -1)
+	    {
+              GtkWidget *error_dialog;
+
+	      error_dialog = gtk_message_dialog_new (NULL,
+                                                     GTK_DIALOG_MODAL,
+						     GTK_MESSAGE_ERROR,
+						     GTK_BUTTONS_CLOSE,
+						     _("Unable to rename file '%s' to '%s': %s"),
+						     data_dir_name,
+						     backup,
+						     g_strerror (errno));
+	      
+	      gtk_dialog_run (GTK_DIALOG (error_dialog));
+	      
+	      gtk_widget_destroy (error_dialog);
+	      g_free (backup);
+	      g_free (data_dir_name);
+
+	      return FALSE;
+            }
+
+	  g_free (backup);
+	  
+          if (g_mkdir (data_dir_name, 0700) == -1)
+            {
+              GtkWidget *error_dialog;
+	      
+	      error_dialog = gtk_message_dialog_new (NULL,
+						     GTK_DIALOG_MODAL,
+						     GTK_MESSAGE_ERROR,
+						     GTK_BUTTONS_CLOSE,
+						     _("Unable to create the data directory '%s': %s"),
+						     data_dir_name,
+						     g_strerror (errno));
+	      
+	      gtk_dialog_run (GTK_DIALOG (error_dialog));
+	      
+	      gtk_widget_destroy (error_dialog);
+              g_free (data_dir_name);
+
+	      return FALSE;
+            }
+
+	  goto success;
+	}
+      
+      if (errno != EEXIST)
+        {
+          GtkWidget *error_dialog;
+
+	  error_dialog = gtk_message_dialog_new (NULL,
+						 GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("Unable to create the data directory '%s': %s"),
+						 data_dir_name,
+						 g_strerror (errno));
+	  
+	  gtk_dialog_run (GTK_DIALOG (error_dialog));
+	  
+	  gtk_widget_destroy (error_dialog);
+	  g_free (data_dir_name);
+
+	  return FALSE;
+	}
+    }
+
+success:
+  g_free (data_dir_name);
+
+  return TRUE;
+}
+
 void
 gdict_init (int *argc, char ***argv)
 {
@@ -292,7 +415,7 @@ gdict_init (int *argc, char ***argv)
   GOptionContext *context;
   GOptionGroup *group;
   gchar *loader_path;
-  gchar *word = NULL;
+  gchar **words = NULL;
   gchar *database = NULL;
   gchar *source_name = NULL;
   gboolean no_window = FALSE;
@@ -300,8 +423,8 @@ gdict_init (int *argc, char ***argv)
 
   const GOptionEntry gdict_app_goptions[] =
   {
-    { "look-up", 0, 0, G_OPTION_ARG_STRING, &word,
-       N_("Word to look up"), N_("word") },
+    { "look-up", 0, 0, G_OPTION_ARG_STRING_ARRAY, &words,
+       N_("Words to look up"), N_("word") },
     { "source", 's', 0, G_OPTION_ARG_STRING, &source_name,
        N_("Dictionary source to use"), N_("source") },
     { "list-sources", 'l', 0, G_OPTION_ARG_NONE, &list_sources,
@@ -323,7 +446,7 @@ gdict_init (int *argc, char ***argv)
   
   singleton = GDICT_APP (g_object_new (GDICT_TYPE_APP, NULL));
   g_assert (GDICT_IS_APP (singleton));
-
+  
   /* create the new option context */
   context = g_option_context_new (_(" - Look up words in dictionaries"));
   
@@ -344,6 +467,14 @@ gdict_init (int *argc, char ***argv)
 					   NULL);
   g_set_application_name (_("Dictionary"));
   
+  if (!gdict_app_create_data_dir ())
+    {
+      g_option_context_free (context);
+      gdict_cleanup ();
+
+      exit (1);
+    }
+  
   /* session management */
   singleton->client = gnome_master_client ();
   if (singleton->client)
@@ -376,8 +507,14 @@ gdict_init (int *argc, char ***argv)
   gdict_source_loader_add_search_path (singleton->loader, loader_path);
   g_free (loader_path);
 
-  if (word)
-    singleton->word = g_strdup (word);
+  if (words)
+    {
+      gsize i;
+      gsize length = g_strv_length (words);
+
+      for (i = 0; i < length; i++)
+	singleton->words = g_list_prepend (singleton->words, g_strdup (words[i]));
+    }
 
   if (database)
     singleton->database = g_strdup (database);
